#ifndef CB__GIFDECODER_H
#define CB__GIFDECODER_H
//
// Copyright (c) 2005 Colosseum Builders, Inc.
// All rights reserved.
//
// Colosseum Builders, Inc. makes no warranty, expressed or implied
// with regards to this software. It is provided as is.
//
// See the README.TXT file that came with this software for restrictions
// on the use and redistribution of this file or send E-mail to
// info@colosseumbuilders.com
//

//
// Description:
//
//   GifDecoder is a class for decoding GIF images into a BitmapImage
//   object.
//
//   This decoder supports the GIF format as defined in
//   "GRAPHICS INTERCHANGE FORMAT", CompuServe, July 31, 1990.
//   Markers of the form #N reference section N of this document.
//
//  Author: John M. Miano - miano@colosseumbuilders.com
//  Date:  June 10, 2005
//

#include "gifinputfilestream.h"
#include "bitmapimage.h"
#include "gif.h"

namespace Colosseum
{

class GifDecoder : public BitmapImageDecoder
{
private:
  GifDecoder (const GifDecoder &) ;
  GifDecoder &operator=(const GifDecoder &) ;

  // LZW Dictionary
  struct DictionaryTree
  {
    UBYTE1 data ;
    unsigned int parent ;
  } ;

#include "beginpackbyte.h"
  struct GifColorTable
  {
    UBYTE1 red, green, blue ;
  } ;
#include "endpackbyte.h"
  GifColorTable color_table [1<<8] ;
  GifColorTable global_color_table [1<<8] ;

  // Values to indicate the presence of a transparent color.
  bool transparent_flag ;
  unsigned int transparent_color ;

  unsigned int global_color_count ;
  unsigned int screen_width ;
  unsigned int screen_height ;
  bool is_interlaced ;
  unsigned int interlace_pass ;

  // Current output position within the image.
  unsigned int row_ptr ;
  unsigned int col_ptr ;

  bool more_images ;
  bool verbose_flag ;
  bool multi_image_mode ;

  // Position of last image on the logical screen.
  unsigned int image_position_top ;
  unsigned int image_position_left ;

  // Compression Data
  unsigned int min_code_size ;
  unsigned int clear_code ;
  UBYTE1 first_character ;
  unsigned int code_size ;
  unsigned int end_code ;
  unsigned int next_code ;
  DictionaryTree lzw_dictionary [1 << GifMaxBitsPerCode] ;

  // We have to save a reference to the input stream
  // because GIF allows multiple images per file.
  GifInputStream *input_stream ;

  // Decompression Functions
  void ReadToNextImage () ;

  void outputToImage (UBYTE1 data, BitmapImage &image) ;
  void outputCode (unsigned int code, BitmapImage &image) ;

  void initialize () ;
  void readToNextImage (GifInputStream &input) ;
  void readStreamHeader (GifInputStream &input) ;
  void reset () ;
  void decompress (BitmapImage &image) ;

  void callProgressFunction (unsigned int pass, unsigned int passcount, unsigned int progress) ;
public:
  GifDecoder () ;
  virtual ~GifDecoder () ;

  void readImage (GifInputStream &inputstream, BitmapImage &image) ;
  void readNextImage (BitmapImage &image) ;
  virtual void readImageFile (const std::string &filename, BitmapImage &image) ;

  //
  void setVerbose (bool state) { verbose_flag = state ; }
  bool getVerbose () const { return verbose_flag ; }

  // In multi-image mode the application can read more than one
  // image from a GIF animation from the input stream.
  bool getMultiImageMode () const { return multi_image_mode ; }
  void setMultiImageMode (bool state) ;
  // In multi-image mode, use this function to determine if the
  // input stream has more images.
  bool moreImages () { return more_images ; }

  // Input stream state functions.
  bool activeInputStream () ;
  void closeInputStream () ;

  // Functions to return the location of the current
  // image within the logical screen.
  unsigned int imagePositionTop () const ;
  unsigned int imagePositionLeft () const ;

  unsigned int screenWidth () const { return screen_width ; }
  unsigned int screenHeight () const { return screen_height ; }

  EXCEPTION(GifError)
} ;

} // END NAMESPACE

#endif
