#include <climits>

#include "gifinputstream.h"
#include "gif.h"


using namespace std ;

namespace Colosseum
{

void GifInputStream::enterBitMode ()
{
  InputByteStream::enterBitMode (0) ;
  block_count = *current_byte ;
  ++ current_byte ;
}

unsigned int GifInputStream::nextCode (unsigned int codesize)
{
  if (codesize < 1 || codesize > GifMaxBitsPerCode)
    throw GifStreamError ("GIF: Internal Error - Invalid Code Size." ) ;

  unsigned int result = 0 ;
  for (unsigned int ii = 0 ; ii < codesize ; ++ ii)
    result |= (nextBit () << ii) ;

  if (result >= (1 << GifMaxBitsPerCode))
    throw GifStreamError ("GIF: Internal Error - Invalid Code." ) ;

  return result ;
}

unsigned int GifInputStream::nextBit ()
{
  static const MASKS [CHAR_BIT] = { 0x1, 0x2, 0x4, 0x8, 0x10, 0x20, 0x40, 0x80 } ;

  if  (bit_position == CHAR_BIT)
  {
    -- block_count ;
    if (block_count == 0)
    {
      advanceCurrentByte () ;
      block_count = *current_byte ;
      if (block_count == 0)
        throw GifStreamError ("GIF: Attempt to read bits beyond compressed data.") ;
    }
    advanceCurrentByte () ;
    bit_position = 0 ;
  }

  unsigned int result ;
  if ((*current_byte & MASKS [bit_position]) == 0)
    result = 0 ;
  else
    result = 1 ;
  ++ bit_position ;
  return result ;
}

void GifInputStream::exitBitMode ()
{
  while (block_count > 0)
  {
    -- block_count ;
    advanceCurrentByte () ;
  }
  in_bit_mode = false ;

  if (getByte () != 0)
    throw GifStreamError ("GIF: Missing terminator after compressed data.") ;
}

} // END NAMESPACE


